
/////////////////////////////////////////////////////////////////////////////////////////
///
///  ========================================
///  Dynamic Links, Version 0.1, January 2003
///  ========================================
///  http://dynamiclinks.mozdev.org/
///  (C) Jiri Boucek, 2002-2003
///
///  NOTE: what you can see is still more a proof of concept than anything else
///        (the code is buggy, not optimized etc. etc.)
///  
///  I WILL APPRECIATE ANY COMMENTS AND SUGGESTIONS
///  (http://dynamiclinks.mozdev.org/feedback.html)
///
/////////////////////////////////////////////////////////////////////////////////////////
///
///  CHANGES
///
///  030124
///  - removed evt.target inefficiency
///  - added special treatment for URLs splitted by <wbr> and <nobr> elements
///
/////////////////////////////////////////////////////////////////////////////////////////
///
///  LIST OF FUNCTIONS
///  =================
///
///  MAIN FUNCTIONS:
///
///  dlDynamicLinks()
///  ----------------
///  - finds and separates piece of text under mouse cursor
///  - modifies it as needed
///  - creates artificial links for selected text
///  - cycles through all alternative links (for both artificial/original links)
///
///  dlKeyUp(), dlMouseOut()
///  -----------------------
///  - both functions "clean" unwanted dynamic links
///
///  AUXILIARY FUNCTIONS:
///
///  dlProcessLinks()
///  - exists just not to repeat the same code two times (for artificial/original links)
///
///  dlAddDerivedLinks()
///  - creates derived links from an absolute URL
///
///  dlTestDomain()
///  - tests if string like "site.org/anydir" should become "http://site.org/anydir"
///  - domain list taken from http://www.iana.org/domain-names.htm
/// 
///  dlRemoveArtificialLink(), dlReturnOriginalLink()
///  - cleaning functions
///
///  dlFindLink(), dlFindLinkObj(), dlFindDocument()
///  - find link URL, link/document object references
///
///  dlRelPathToAbsPath()
///  - converts relative path to absolute path
///
///  dlPrefsInit()
///  - initializes preferences in pref.js
///
///  dlStop()
///  - returns info whether dynamiclinks should be stopped
///
///  dlRemoveSelection()
///  - removes stored selection string
///
/////////////////////////////////////////////////////////////////////////////////////////

/// array _elementIDs comes from dynamiclinks-prefs-aux.xul
var _elementIDs = ["dlOnoff", "dlMousemoves",
                   "dl1a", "dl2a", "dl3a", "dl4a", "dl5a", "dl6a", "dl7a", "dl8a", "dl9a",
                   "dl1b", "dl2b", "dl3b", "dl4b", "dl5b", "dl6b", "dl7b", "dl8b", "dl9b" ];

var _elementIDs_types =
                  ["Bool", "Int",
                   "Bool", "Bool", "Bool", "Bool", "Bool", "Bool", "Bool", "Bool", "Bool",
                   "Char", "Char", "Char", "Char", "Char", "Char", "Char", "Char", "Char" ];

var _elementIDs_values =
                  [true, 12,
                   true, false, true, true, true, true, false, false, false,
                   "http://$$$",
                   "http://$$$.org",
                   "http://www.google.com/search?q=$$$",
                   "http://news.google.com/news?q=$$$",
                   "http://groups.google.com/groups?q=$$$",
                   "http://groups.google.com/groups?q=$$$&meta=group%3Dnetscape.public.mozilla.*",
                   "http://www.m-w.com/cgi-bin/dictionary?book=Dictionary&va=$$$",
                   "", 
                   "http://<FILL IN WHATEVER YOU WANT TO USE PLUS $$$>" ];

function dlDynamicLinks(evt) {
  /// check if dynamiclinks should be active
  if (dlStop()) return;

  /// make it frames-friendly => find document for given element
  if (!(doc=dlFindDocument(evt.target))) return;

  /// there are two beasts to work with - artificial link or original link
  var artificialLinkElem = doc.getElementById('artificialLinkElem');
  var originalLinkElem = doc.getElementById('originalLinkElem');

  /// end if Ctrl not pressed (remove any existing artificial dynamic link,
  ///  return back original link URL, delete saved selection string)
  if (!evt.ctrlKey) {
    if (artificialLinkElem) dlRemoveArtificialLink(doc, artificialLinkElem);
    if (originalLinkElem) dlReturnOriginalLink(originalLinkElem);
    dlRemoveSelection();
    return;
  }

  /// remove every artificial dynamic link outside mouse focus
  if (artificialLinkElem && (artificialLinkElem!=dlFindLinkObj(evt.target)) ) {
    dlRemoveArtificialLink(doc, artificialLinkElem);
    return;
  }
  /// same for every changed original link outside mouse focus
  if (originalLinkElem && (originalLinkElem!=dlFindLinkObj(evt.target)) ) {
    dlReturnOriginalLink(originalLinkElem);
    return;
  }

  /// save selection if it exists (selection as in window.getSelection())
  try {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    if (doc==focusedWindow.document) {
      var selectedText = focusedWindow.getSelection().toString();
      if (selectedText && !focusedWindow.document.getElementsByTagName('head')[0].hasAttribute('dlselection')) {
        selectedText = selectedText.replace(/^[ \n\t\r]+/,'');
        selectedText = selectedText.replace(/[ \n\t\r]+$/g,'');
        selectedText = selectedText.replace(/[ \n\t\r]+/g,' ');
        var dlIsLink=false;
        if (selectedText.toLowerCase().search(/^[\w-\.]+(\.[a-z]+)($|\/)/)!=-1) {
          if (dlTestDomain(RegExp.$1)) dlIsLink=true;
        }
        if (selectedText.search(/(https?|ftp):\/\//)==0 || dlIsLink) { 
          selectedText = selectedText.replace(/[ \n\t\r]/g,'');
        } else {
          selectedText = selectedText.replace(/[ \n\t\r]/g,'+');
        }
        focusedWindow.document.getElementsByTagName('head')[0].setAttribute('dlselection', selectedText);
      }
    }
  } catch(e) {}

  ///
  /// CREATE ARTIFICIAL LINK
  ///
  /// repeatedly divide DOM #text node (using DOM splitText)
  ///

  if (evt.target.nodeType==3 && !dlFindLink(evt.target) ) { 
    var tar=evt.target;
    var lastParent=tar.parentNode;
    var str=tar.nodeValue;
    var len = str.length; 
    var half=Math.floor(len/2);
    var canContinue=true;
    var pos;

    /// this is the split algorithm
    var chars = new Array(' ', '"', "'", '>', '<', '\n', '\r');
    for (var i=0; i<chars.length; i++) {
      if (str!=chars[i]) {
        pos=str.slice(half,len).indexOf(chars[i]);
        if (pos!=-1) {
          tar.splitText(half+pos); 
          canContinue=false;
        }
        pos=str.slice(0,half).lastIndexOf(chars[i]);
        if (pos!=-1) {
          tar.splitText(pos+1); 
          canContinue=false;
        }
      }
      if (!canContinue) return;
    }

    /// no whitespaces
    pos = str.search(/[\s\]\[]/);
    if (pos!=-1) {
      tar.splitText(pos+1);
      tar.splitText(pos);
      return;
    }

    /// more pre-processing (esp. for pages viewed through 'view-source:')
    pos = str.search(/(&lt;|&gt;|&quot;|]]|!\[CDATA\[))/);
    if (pos!=-1) {
      var found = RegExp.$1;
      tar.splitText(pos+found.length);
      tar.splitText(pos);
      return;
    }

    /// remove last char
    if ( str.substr(-1).search(/[.,;:)(\]&\?]/) != -1 ) {
      tar.splitText(str.length-1);
      return;
    }

    /// remove first char
    if ( str.substr(0,1).search(/[@()]/) != -1 ) {
      tar.splitText(1);
      return;
    }

    /// create artificial link
    if (str.length>1) {                       //if (str.indexOf(' ')==-1) { 
      var elem = doc.createElement('a');
      elem.appendChild(doc.createTextNode(str));
      elem.setAttribute('id','artificialLinkElem');
      elem.setAttribute('temporarilyReplacedText',str);

      var index = 1;
      var URL = doc.location.href;

      /// use selection if it exists
      try {
        if (focusedWindow.document.getElementsByTagName('head')[0].hasAttribute('dlselection')) {
          var selectedText = focusedWindow.document.getElementsByTagName('head')[0].getAttribute('dlselection');
          str = selectedText;
        }
      } catch(e) {}

      /// 
      /// WORDS WITHOUT DOTS/SLASHES ARE PROCESSED HERE
      ///

      if (str.search(/[\.\/]/)==-1) {
        // general word - use it for lookups
        var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
        var requestedURL;
        for (var i=1; i<10; i++) {
          requestedURL = mPrefs.getCharPref('extensions.dynamiclinks.dl'+i+'b');
          if (!requestedURL || requestedURL.indexOf('$$$')==-1) continue;
          if (mPrefs.getBoolPref('extensions.dynamiclinks.dl'+i+'a')) {
            if (selectedText && requestedURL.indexOf('google')==-1) {
              // any web sites to supply with multiple selected words?
            } else {
              elem.setAttribute(index++, requestedURL.replace(/\$\$\$/, str));
              // clear the selection ?
            }
          }
        }

        // TO DO:
        // - add amazon ? (HTTP POST)
        // - context: isbn numbers etc.
        // - process two http's in the link (http://site1/dir?whatever=http://site2)

        if (URL.indexOf('view-source:')==0) {
          // can it be relative URLs ?
          str = dlRelPathToAbsPath(URL.slice('view-source:'.length), str);
        }
      }

      if (URL.indexOf('view-source:')==0) URL = URL.slice('view-source:'.length);

      dlProcessLinks(doc, elem, str, URL, index, evt);

      /// replace the original text => create the link
      lastParent.replaceChild(elem, tar);
    }

  }

  /// 
  /// MODIFY ORIGINAL LINKS
  ///

  var elem = dlFindLinkObj(evt.target);
  if (evt.target.nodeType==3 && elem && elem!=artificialLinkElem && elem!=originalLinkElem) { 
    if (elem.hasAttribute('id')) elem.setAttribute('origId', elem.getAttribute('id'));
    elem.setAttribute('id','originalLinkElem');
    var str = elem.getAttribute('href');
    elem.setAttribute('temporarilyReplacedText', str);
    var index = 1;
    var URL = doc.location.href;
    dlProcessLinks(doc, elem, str, URL, index, evt);
  }

  ///
  /// CYCLE THROUGH LINKS (works with both artificial/original links)
  ///

  if (elem) {
    /// mousemoves => try next link
    var dist = elem.getAttribute('dist');
    /// change the dist value up or down to tune sensitivity 
    ///  (the distance mouse has to run to activate the next link)
    var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
    var distMax = mPrefs.getIntPref('extensions.dynamiclinks.dlMousemoves');
    if (dist > distMax) {
      elem.setAttribute('dist', 0);
      var index = elem.getAttribute('index');
      var nextLink = elem.getAttribute(index);
      if (nextLink) {
        elem.setAttribute('href', nextLink);
        try {
          document.getAnonymousNodes(document.getElementById('statusbar-display'))[0].setAttribute('value', nextLink);
        } catch(e) {}
      }
      // WHOLE LINK elem.firstChild.nodeValue = elem.getAttribute(index++);
      index++;
      if (!elem.hasAttribute(index)) index=1;
      elem.setAttribute('index', index);
    } else {
      elem.setAttribute('dist', ++dist);
    }
  }

}    /// end of function dlDynamicLinks(evt)

/////////////////////////////////////////////////////////////////////////////////////////

function dlProcessLinks(doc, elem, str, URL, index, evt) {

  /// process email addresses (add 'mailto:' if needed and optionally use just host)
  if (str.search(/([^:@]+@)(.+)(\.[^\?\n\r\t]+)(\?[^\n\r\t]+)?/i)!=-1) {
    if (dlTestDomain(RegExp.$3) && str.indexOf('://')==-1) {
      elem.setAttribute(index++, 'mailto:' + RegExp.$1 + RegExp.$2 + RegExp.$3 + RegExp.$4);
      if (RegExp.$4) elem.setAttribute(index++, 'mailto:' + RegExp.$1 + RegExp.$2 + RegExp.$3);
      str = 'http://' + RegExp.$2 + RegExp.$3 + '/';
    }
  }

  /// try to extract links from URLs with "javascript:" scheme (very simplistic right now)
  if (str.search(/javascript:[^'"]+('[^']+'|"[^"]+")/i)==0) {
    str = RegExp.$1;
    if (str.search(/[\.\/]/)!=-1) {
      str = str.slice(1,-1);
    } else {
      str = '';
    }
  }
  
  /// process relative URLs (dlTestDomain() determines whether a path is really relative or not)
  if (str.search(/(http|ftp):\/\//i)!=0 && str.search(/[\.\/#]/)!=-1 && str.toLowerCase().indexOf('file:')!=0) {
    if (str.search(/^([^\/]+)/)!=-1) {
      var host = RegExp.$1;
      var mystr = host.slice(host.lastIndexOf('.'));
      if (dlTestDomain(mystr)) {
        if (str.toLowerCase().indexOf('ftp.')==0) {
          str = 'ftp://' + str;
        } else {
          str = 'http://' + str;
        }
      } else {
        str = dlRelPathToAbsPath(URL, str);
      }
    } else {
      str = dlRelPathToAbsPath(URL, str);
    }
  }
  
  /// process absolute URLs
  if (str.search(/(https?|ftp):\/\//i) == 0) {

    // but first treat <wbr> and <nobr> elements
    var tar = evt.target;

    // decafbad.com example: http://diveintopython.org/dialect_divein.h<wbr>tml
    try {
      var sib = tar.nextSibling;
      if (sib && sib.nodeName.toLowerCase()=='wbr') {
        var sib = sib.nextSibling;
        if (sib && sib.nodeType==3) {
          if (sib.nodeValue.search(/^([^ \n\r"'\]>]+)/)!=-1) str += RegExp.$1;
        }
      }
    } catch(e) {}

    // slashdot.org example: "http://grunge.cs.tu-berlin.de/~tolk/vmlanguages.h<nobr>t<wbr></nobr> ml"
    try {
      var sib = tar.nextSibling;
      if (sib && sib.nodeName.toLowerCase()=='nobr') {
        var chld = sib.firstChild;
        var appstr = chld.nodeValue;
        if (chld && chld.nodeType==3 && appstr.length==1 && appstr.search(/[ \n\r]/)==-1) {
          if (chld.nextSibling && chld.nextSibling.nodeName.toLowerCase()=='wbr' 
               && sib.nextSibling && sib.nextSibling.nodeType==3) {
            if (sib.nextSibling.nodeValue.search(/^ ([^ \n\r"'\]>]+)/)!=-1) str += appstr + RegExp.$1;
          }
        }
      }
    } catch(e) {}


    if (str.indexOf('/', str.indexOf('://')+3)==-1) { 
      if (str.slice(-1)=='.') str = str.slice(0,-1);
      str += '/';
    }
    dlAddDerivedLinks(doc, elem, str, index);
  }

  if (elem.hasAttribute('1')) {
    try {
      /// remember window status information (to return later; more reliable than using window.status)
      elem.setAttribute('originalWindowStatus', document.getAnonymousNodes(document.getElementById('statusbar-display'))[0].getAttribute('value'));
    } catch(e) {}        
    elem.setAttribute('href', elem.getAttribute('1'));
    try {
      document.getAnonymousNodes(document.getElementById('statusbar-display'))[0].setAttribute('value', elem.getAttribute('href'));
    } catch(e) {}
  }

  elem.setAttribute('dist', 0);
  elem.setAttribute('index',2);
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlAddDerivedLinks(doc, elem, url, index) {
  var pos=url.length-1;
  var hostStartPos=0;
  if (url.indexOf('://')) hostStartPos = url.indexOf('://') + 3;
  var hostEndPos = url.indexOf('/', hostStartPos);

  if (!elem.hasAttribute('href')) elem.setAttribute('href', url);
  elem.setAttribute(index++, url);

  /// cut reference portion of URL
  if ((pos=url.lastIndexOf('#'))>-1) elem.setAttribute(index++, url.slice(0,pos));

  /// cut query portion of URL
  while ((pos=url.slice(0,pos).lastIndexOf('&'))>-1) elem.setAttribute(index++, url.slice(0,pos));
  if ((pos=url.lastIndexOf('?'))>-1) elem.setAttribute(index++, url.slice(0,pos));

  /// cut parameters portion of URL
  if ((pos=url.lastIndexOf(';'))>-1) elem.setAttribute(index++, url.slice(0,pos));

  /// cut path segments of URL
  while ((pos=url.slice(0,pos).lastIndexOf('/'))>-1 && pos>=hostEndPos && hostEndPos!=-1) 
    elem.setAttribute(index++, url.slice(0,pos) + '/');

  var scheme;
  pos = 0;

  /// cut/change host segments of URL
  while ((pos=url.indexOf('.', pos))>-1 && pos<url.slice(0,hostEndPos).lastIndexOf('.')) {
    var host = url.slice(pos+1, hostEndPos+1);
    if (host.toLowerCase().indexOf('ftp.')==0) { scheme = 'ftp://'; } else { scheme = 'http://'; }
    //scheme = url.slice(0,hostStartPos);
    elem.setAttribute(index++, scheme + host);
    pos++;
  }

  try {
    if (url.toLowerCase().indexOf('www')==-1) {
      if (host) {
        if (host.toLowerCase().indexOf('ftp.')==0) { scheme = 'ftp://'; } else { scheme = 'http://'; }
        elem.setAttribute(index++, scheme + 'www.' + host);
      } else {
        var wwwLink = elem.getAttribute(index-1);
        elem.setAttribute(index++, wwwLink.replace(/:\/\//, '://www.'));
      }
    }
  } catch(e) {}
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlTestDomain(str) {
  var domainList = new Array('aero', 'biz', 'com', 'coop', 'info', 'museum', 'name', 'net', 'org', 'pro', 'gov', 'edu', 'mil', 'int', 'al', 'dz', 'as', 'ad', 'ao', 'ai', 'aq', 'ag', 'ar', 'am', 'aw', 'au', 'at', 'az', 'bs', 'bh', 'bd', 'bb', 'by', 'be', 'bz', 'bj', 'bm', 'bt', 'bo', 'ba', 'bw', 'bv', 'br', 'io', 'bn', 'bg', 'bf', 'bi', 'kh', 'cm', 'ca', 'cv', 'ky', 'cf', 'td', 'cl', 'cn', 'cx', 'cc', 'co', 'km', 'cg', 'cd', 'ck', 'cr', 'ci', 'hr', 'cu', 'cy', 'cz', 'dk', 'dj', 'dm', 'do', 'ec', 'eg', 'sv', 'gq', 'er', 'ee', 'et', 'fk', 'fo', 'fj', 'fi', 'fr', 'gf', 'pf', 'tf', 'ga', 'gm', 'ge', 'de', 'gh', 'gi', 'gr', 'gl', 'gd', 'gp', 'gu', 'gt', 'gn', 'gw', 'gy', 'ht', 'hm', 'va', 'hn', 'hk', 'hu', 'is', 'in', 'id', 'ir', 'iq', 'ie', 'il', 'it', 'jm', 'jp', 'jo', 'kz', 'ke', 'ki', 'kp', 'kr', 'kw', 'kg', 'la', 'lv', 'lb', 'ls', 'lr', 'ly', 'li', 'lt', 'lu', 'mo', 'mk', 'mg', 'mw', 'my', 'mv', 'ml', 'mt', 'mh', 'mq', 'mr', 'mu', 'yt', 'mx', 'fm', 'md', 'mc', 'mn', 'ms', 'ma', 'mz', 'mm', 'na', 'nr', 'np', 'nl', 'an', 'nc', 'nz', 'ni', 'ne', 'ng', 'nu', 'nf', 'mp', 'no', 'om', 'pk', 'pw', 'ps', 'pa', 'pg', 'py', 'pe', 'ph', 'pn', 'pl', 'pt', 'pr', 'qa', 're', 'ro', 'ru', 'rw', 'sh', 'kn', 'lc', 'pm', 'vc', 'ws', 'sm', 'st', 'sa', 'sn', 'sc', 'sl', 'sg', 'sk', 'si', 'sb', 'so', 'za', 'gs', 'es', 'lk', 'sd', 'sr', 'sj', 'sz', 'se', 'ch', 'sy', 'tw', 'tj', 'tz', 'th', 'tl', 'tg', 'tk', 'to', 'tt', 'tn', 'tr', 'tm', 'tc', 'tv', 'ug', 'ua', 'ae', 'gb', 'uk', 'us', 'um', 'uy', 'uz', 'vu', 've', 'vn', 'vg', 'vi', 'wf', 'eh', 'ye', 'yu', 'zm', 'zw');

  for (var i=0; i<domainList.length; i++) {
    if (str.toLowerCase() == '.'+domainList[i]) return '.'+domainList[i];
  }
  return false;
}
  
/////////////////////////////////////////////////////////////////////////////////////////

function dlKeyUp(evt) {
  /// this removes any dynamic link when the ctrlKey is released
  if (evt.charCode==0 && evt.keyCode==17) {
    if (dlStop()) return;
    if (!(doc=dlFindDocument(evt.target))) return;
    var artificialLinkElem = doc.getElementById('artificialLinkElem');
    if (artificialLinkElem) dlRemoveArtificialLink(doc, artificialLinkElem);
    var originalLinkElem = doc.getElementById('originalLinkElem');
    if (originalLinkElem) dlReturnOriginalLink(originalLinkElem);
    dlRemoveSelection();
  }
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlMouseOut(evt) {
  /// this removes any dynamic link when the mouse cursor leaves the text node
  if (dlStop()) return;
  if (!(doc=dlFindDocument(evt.target))) return;
  var tar = evt.target;
  if (tar.nodeType==3) {
    var artificialLinkElem = doc.getElementById('artificialLinkElem');
    if (tar.parentNode==artificialLinkElem) dlRemoveArtificialLink(doc, artificialLinkElem);
    var originalLinkElem = doc.getElementById('originalLinkElem');
    if (tar.parentNode==originalLinkElem) dlReturnOriginalLink(originalLinkElem);
    // dlRemoveSelection(doc);
  }
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlRemoveArtificialLink(doc, elem) {
  // "doc" is document containing target node
  try {
    document.getAnonymousNodes(document.getElementById('statusbar-display'))[0].setAttribute('value', elem.getAttribute('originalWindowStatus')); 
  } catch(e) {}
  var txt = elem.getAttribute('temporarilyReplacedText');
  if (!txt) txt = elem.firstChild.nodeValue;     // ?
  var txtNode = doc.createTextNode(txt);
  var par = elem.parentNode;
  par.replaceChild(txtNode, elem);
  par.normalize();
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlReturnOriginalLink(elem) {
  try {
    if (elem.hasAttribute('originalWindowStatus')) {
      document.getAnonymousNodes(document.getElementById('statusbar-display'))[0].setAttribute('value', elem.getAttribute('originalWindowStatus')); 
      elem.removeAttribute('originalWindowStatus');
    }
  } catch(e) {}
  if (elem.hasAttribute('temporarilyReplacedText')) { 
    elem.setAttribute('href', elem.getAttribute('temporarilyReplacedText'));
    elem.removeAttribute('temporarilyReplacedText');
  }
  if (elem.hasAttribute('origId')) {
    elem.setAttribute('id', elem.getAttribute('origId'));
    elem.removeAttribute('origId');
  } else {
    elem.removeAttribute('id');
  }
  if (elem.hasAttribute('index')) elem.removeAttribute('index');
  if (elem.hasAttribute('dist')) elem.removeAttribute('dist');
/* CLEAN ALL CREATED LINK ATTRIBUTES?
  var index = 1;
  while (elem.hasAttribute(index)) {
    elem.removeAttribute(index);
    index++
  }
*/
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlFindLink(obj) {
  var node = obj;
  while (node) {
    if (node.nodeName == 'A' && node.hasAttribute('href')) return node.getAttribute('href');
    node = node.parentNode;
  }
  return null;
}

function dlFindLinkObj(obj) {
  var node = obj;
  while (node) {
    if (node.nodeName == 'A' && node.hasAttribute('href')) return node;
    node = node.parentNode;
  }
  return null;
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlFindDocument(target) {
  var node = target;
  var saved = '';
  while (node) {
    if (node.nodeType == Node.DOCUMENT_NODE) {
      //  skip XUL documents (and right now also XML documents)
      if (node.toString().search(/XML|XUL/)!=-1) return '';
      return node;
    }
    node = node.parentNode;
    if (node) saved = node;
  }
  return saved;
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlRelPathToAbsPath(ref,rel) {
  var uri = Components.classes["@mozilla.org/network/standard-url;1"].createInstance(Components.interfaces.nsIURI);
  uri.spec = ref;
  return uri.resolve(rel);
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlPrefsInit() {
  var prefBase = 'extensions.dynamiclinks.';
  var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  for (var i=0; i<_elementIDs.length; i++) {
    if (!mPrefs.getPrefType(prefBase+_elementIDs[i])) { 
      eval("mPrefs.set" + _elementIDs_types[i] + "Pref(prefBase+_elementIDs[i], _elementIDs_values[i]);");
    } 
  }
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlStop() {
  var mPrefs = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
  return !mPrefs.getBoolPref('extensions.dynamiclinks.dlOnoff');
}

/////////////////////////////////////////////////////////////////////////////////////////

function dlRemoveSelection() {
  try {
    var focusedWindow = document.commandDispatcher.focusedWindow;
    if (focusedWindow.document.getElementsByTagName('head')[0].hasAttribute('dlSelection')) 
      focusedWindow.document.getElementsByTagName('head')[0].removeAttribute('dlSelection');
  } catch(e) {}
}

/////////////////////////////////////////////////////////////////////////////////////////

